import unified from 'unified';
import markdown from 'remark-parse';
import remark2rehype from 'remark-rehype';
import rehype2react from 'rehype-react';

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

var rComponent = /^@\[(\w+)\]\(([^()]*)\)/;

function tokenizeComponent(eat, value, silent) {
  var match = rComponent.exec(value);
  if (!match) return null;
  if (silent) return true;

  var _match = _slicedToArray(match, 3),
      all = _match[0],
      component = _match[1],
      props = _match[2];

  return eat(all)({
    type: 'component',
    component: component,
    props: props
  });
}

tokenizeComponent.locator = function (value, fromIndex) {
  return value.indexOf('@[', fromIndex);
};

function parseComponents() {
  var _Parser$prototype = this.Parser.prototype,
      inlineTokenizers = _Parser$prototype.inlineTokenizers,
      inlineMethods = _Parser$prototype.inlineMethods;
  inlineTokenizers.component = tokenizeComponent;
  inlineMethods.splice(inlineMethods.indexOf('link'), 0, 'component');
}

var index = {
  name: 'VueRemark',
  functional: true,
  inject: {
    mdComponents: {
      default: {}
    }
  },
  render: function render(createElement, context) {
    var _context$slots$defaul = _slicedToArray(context.slots().default, 1),
        slot = _context$slots$defaul[0];

    var mdComponents = context.injections.mdComponents;
    var handlers = {
      component: function component(h, node) {
        var mdComponent = mdComponents[node.component];
        if (typeof mdComponent === 'function') return mdComponent(h, node);
        if (typeof mdComponent === 'string') return h(node, mdComponent);

        var _Object = Object(mdComponent),
            props = _Object.props;

        return h(node, node.component, {
          props: typeof props === 'function' ? props(node.props) : null
        });
      }
    };
    var components = Object.keys(mdComponents).reduce(function (components, name) {
      var component = mdComponents[name].component;
      if (component) components[name] = component;
      return components;
    }, {});
    return unified().use(markdown).use(parseComponents).use(remark2rehype, {
      handlers: handlers
    }).use(rehype2react, {
      createElement: createElement,
      components: components,
      prefix: false
    }).processSync(slot.text).contents;
  }
};

export default index;
